classdef LmApi
    %lmApi Class to access LineMetrics RESTful API
    %   This class contains methods to access the LineMetrics RESTful API.
    %   At the moment only the method to access historic data is
    %   implemented.
    %
    %   If you have any questions or ideas, don't hestiate to contact the
    %   author.
    %
    %   Wolfgang Lubowski, LineMetrics GmbH
    %   w.lubowski@linemetrics.com
    %   August 2019
    
    properties
        ClientId
        ClientSecret
        AccessToken
        ApiOptions
    end
    
    methods
        function obj = LmApi(id, secret)
            %LmApi Construct an instance of this class
            %   write client id and client secret to properties and
            %   retrieve an access token
            obj.ClientId = id;
            obj.ClientSecret = secret;
            obj.AccessToken = obj.retrieveToken(obj.ClientId, ...
                obj.ClientSecret);
            obj.ApiOptions = obj.setApiOptions(obj.AccessToken);
        end
        
        function data = loadData(obj, dateFrom,dateTo, objectId, apiAlias)
            %loadData 
            %   inputs:
            %   dateFrom: in fromat yyyy-MM-dd HH:mm (e.g. 2019-08-05 17:37)
            %   dateTo: in format yyyy-MM-dd HH:mm
            %   objectId: ID of LineMetrics cloud object, can be found in
            %   the cloud in the measurement point view when you click on
            %   "API" in the upper right corner, it is an 32 digit hash,
            %   alternatively you can also use the object alias, which is
            %   the object name
            %   apiAlias: the alias you defined for the desired measurement
            %   point
            date1 = num2str(posixtime(datetime(dateFrom,'InputFormat',...
                'yyyy-MM-dd HH:mm')) * 1e3);
            date2 = num2str(posixtime(datetime(dateTo,'InputFormat',...
                'yyyy-MM-dd HH:mm')) * 1e3);
            dataRaw = webread(['https://rest-api.linemetrics.com/v2/data/'...
                objectId '/' apiAlias '?function=raw&time_from=' date1...
                '&time_to=' date2],obj.ApiOptions);
            dataRawC = struct2cell(dataRaw);
            valC = dataRawC(1,:);
            tsC = dataRawC(2,:);
            emptyIndicesV = cellfun(@isempty,valC);
            emptyIndicesT = cellfun(@isempty,tsC);
            valC(emptyIndicesV)={[0]};
            tsC(emptyIndicesT)={[0]};
            valA=cell2mat(valC);
            tsA=cell2mat(tsC);
            valA(emptyIndicesV)=NaN;
            tsA(emptyIndicesT)=NaN;
            data = timeseries(valA,datestr(datetime(tsA/1000,...
                'ConvertFrom', 'posixtime' )));
            data.name=[objectId '/' apiAlias];
        end
    end
    
    methods (Access = private)
        function accessToken = retrieveToken(~, id, secret)
            %retrieveToken retrieves an access token from the LM REST API
            %   attention: the token expires after one hour
            response = webwrite('https://rest-api.linemetrics.com/oauth/access_token', ...
                ['client_id=' id '&grant_type=client_credentials&client_secret=' secret]);
            accessToken = response.access_token;
            disp('LineMetrics API access token has been retrieved.')
            disp('This token is expiring in one hour.')
            disp('Then you have to get a new token.')
        end
        
        function apiOptions = setApiOptions(~,accessToken)
            %setApiOptions
            %   save api access token into a weboptions object, that will
            %   be used in any further connection with the api
            headerFields = {'Authorization', ['Bearer ', accessToken]};
            apiOptions = weboptions('HeaderFields', headerFields, 'ContentType','json');
        end
    end
end

